searchData={"items":[{"type":"module","title":"Membrane.WAV.Parser","doc":"Element responsible for parsing WAV files.\n\nIt requires WAV file in uncompressed, PCM format on the input (otherwise error is raised) and\nprovides raw audio on the output. WAV header is parsed to extract metadata of the raw audio format.\nThen it is dropped and only samples are sent to the next element.\n\nThe element has one option - `frames_per_buffer`. User can specify number of frames sent in\none buffer when demand unit on the output is `:buffers`. One frame contains\n`bits per sample * number of channels` bits.","ref":"Membrane.WAV.Parser.html"},{"type":"module","title":"WAV Header - Membrane.WAV.Parser","doc":"```\n   0                   4                   8                   12                  16\n   _________________________________________________________________________________\n0  |                   |                   |                   |                   |\n   |      \"RIFF\"       |    file length    |      \"WAVE\"       |       \"fmt \"      |\n   |                   |                   |                   |                   |\n   |___________________|___________________|___________________|___________________|\n16 |                   |         |         |                   |                   |\n   |   format chunk    | format  |number of|      sample       | data transmission |\n   |      length       |(1 - PCM)|channels |       rate        |       rate        |\n   |___________________|_________|_________|___________________|___________________|\n32 |  block  |  bits   |                   |                   |                   |\n   |  align  |  per    |      \"fact\"       |     fact chunk    |    samples per    |\n   |  unit   | sample  |                   |       length      |      channel      |\n   |_________|_________|___________________|___________________|___________________|\n48 |                   |                   |                                       |\n   |      \"data\"       |    data length    |                 DATA                  |\n   |                   |     in bytes      |                                       |\n   |___________________|___________________|_______________________________________|\n```\nHeader may contain additional bytes between `bits per sample` and `\"fact\"` in case of `format`\ndifferent from 1 (1 represents PCM / uncompressed format). Length of block from `format` until\n`\"fact\"` is present in `format chunk length` (it is 16 for PCM).\n\nBlocks from byte 36 to 48 are optional. There can be additional bytes after `samples per\nchannel` if `fact chunk length` contains number bigger than 4.","ref":"Membrane.WAV.Parser.html#module-wav-header"},{"type":"module","title":"Parsing - Membrane.WAV.Parser","doc":"Stages of parsing:\n- `:init` - Parser waits for the first 22 bytes. After getting them, it parses these bytes\n  to ensure that it is a WAV file. Parser knows `format chunk length` and `format`, so it\n  is able to raise an error in case of different `format` than 1 (PCM) or different\n  length than 16 (for PCM). After parsing, the stage is set to `:format`.\n- `:format` - Parser waits for the next 22 bytes - `fmt` chunk (bytes 20 - 35) without\n  `format` and either `\"fact\"` and `fact chunk length` or `\"data\"` and `data length in bytes`.\n  Then it parses it and create `Membrane.RawAudio` struct with audio format to send it\n  as stream_format to the next element. Stage is set to `:fact` or `:data` depending on last 8 bytes.\n- `:fact` - Parser waits for `8 + fact chunk length` bytes. It  parses them only to check if\n  the header is correct, but does not use that data in any way. After parsing, the stage is\n  set to `:data`.\n- `:data` - header is already fully parsed. All new data from the input is sent to the output.","ref":"Membrane.WAV.Parser.html#module-parsing"},{"type":"module","title":"Pads - Membrane.WAV.Parser","doc":"### `:input`\n\nAccepted formats:\n```\nRemoteStream\n```\n\nDirection: | `:input`\nAvailability: | `:always`\nFlow control: | `:auto`\nDemand unit: | `:bytes`\n\n\n### `:output`\n\nAccepted formats:\n```\nRawAudio\n```\n\nDirection: | `:output`\nAvailability: | `:always`\nFlow control: | `:auto`\nDemand unit: | `nil`","ref":"Membrane.WAV.Parser.html#module-pads"},{"type":"module","title":"Membrane.WAV.Postprocessing","doc":"Module responsible for post-processing serialized WAV files.\n\nDue to the fact that `Membrane.WAV.Serializer` with seeking disabled creates WAV file with\nincorrect `file length` and `data length` blocks in the header, post-processing is needed.\n`fix_wav_header/1` fixes that problem.\n\nHeader description can be found in `Membrane.WAV.Parser`.","ref":"Membrane.WAV.Postprocessing.html"},{"type":"function","title":"Membrane.WAV.Postprocessing.fix_wav_header/1","doc":"Fixes header of the WAV file located in `path`.","ref":"Membrane.WAV.Postprocessing.html#fix_wav_header/1"},{"type":"type","title":"Membrane.WAV.Postprocessing.fix_error/0","doc":"","ref":"Membrane.WAV.Postprocessing.html#t:fix_error/0"},{"type":"module","title":"Membrane.WAV.Serializer","doc":"Element responsible for raw audio serialization to WAV format.\n\nCreates WAV header (its description can be found with `Membrane.WAV.Parser`) based on a format received in stream_format and puts it before audio samples. The element assumes that audio is in PCM format.\n\n`file length` and `data length` fields can be calculated only after processing all samples, so\nthe serializer uses `Membrane.File.SeekEvent` to supply them with proper values before the end\nof stream. If your sink doesn't support seeking, set `disable_seeking` option to `true` and fix\nthe header using `Membrane.WAV.Postprocessing`.","ref":"Membrane.WAV.Serializer.html"},{"type":"module","title":"Element options - Membrane.WAV.Serializer","doc":"Passed via struct `t:Membrane.WAV.Serializer.t/0`\n\n- `disable_seeking`  \n\n  ```\n  boolean()\n  ```\n  \n  Default value: `false`  \n  Whether the element should disable emitting `Membrane.File.SeekEvent`.\n  \n  The event is used to supply the WAV header with proper values before\n  the end of stream. If your sink doesn't support it, you should set this\n  option to `true` and use `Membrane.WAV.Postprocessing` to fix the header.","ref":"Membrane.WAV.Serializer.html#module-element-options"},{"type":"module","title":"Pads - Membrane.WAV.Serializer","doc":"### `:input`\n\nAccepted formats:\n```\nRawAudio\n```\n\nDirection: | `:input`\nAvailability: | `:always`\nFlow control: | `:auto`\nDemand unit: | `:bytes`\n\n\n### `:output`\n\nAccepted formats:\n```\n_any\n```\n\nDirection: | `:output`\nAvailability: | `:always`\nFlow control: | `:auto`\nDemand unit: | `nil`","ref":"Membrane.WAV.Serializer.html#module-pads"},{"type":"function","title":"Membrane.WAV.Serializer.options/0","doc":"Returns description of options available for this module","ref":"Membrane.WAV.Serializer.html#options/0"},{"type":"type","title":"Membrane.WAV.Serializer.t/0","doc":"Struct containing options for `Membrane.WAV.Serializer`","ref":"Membrane.WAV.Serializer.html#t:t/0"},{"type":"extras","title":"Membrane WAV Plugin","doc":"# Membrane WAV Plugin\n\n[![Hex.pm](https://img.shields.io/hexpm/v/membrane_wav_plugin.svg)](https://hex.pm/packages/membrane_wav_plugin)\n[![API Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_wav_plugin)\n[![CircleCI](https://circleci.com/gh/membraneframework/membrane_wav_plugin.svg?style=svg)](https://circleci.com/gh/membraneframework/membrane_wav_plugin)\n\nPlugin providing elements handling audio in WAV file format.\n\nIt is part of [Membrane Multimedia Framework](https://membraneframework.org).","ref":"readme.html"},{"type":"extras","title":"Installation - Membrane WAV Plugin","doc":"The package can be installed by adding `membrane_wav_plugin` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n\t  {:membrane_wav_plugin, \"~> 0.9.1\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Parser - Membrane WAV Plugin","doc":"The Parser requires a WAV file on the input and provides a raw audio in uncompressed, PCM format on\nthe output.\n\nParsing steps:\n\n- Reading WAV header\n- Extracting audio metadata and sending it through a pad to the next element\n- Sending only audio samples to the next elements\n\nIt can parse only uncompressed audio.","ref":"readme.html#parser"},{"type":"extras","title":"Serializer - Membrane WAV Plugin","doc":"The Serializer adds WAV header to the raw audio in uncompressed, PCM format.\n\n#","ref":"readme.html#serializer"},{"type":"extras","title":"Warning - Membrane WAV Plugin","doc":"The Serializer creates header with invalid blocks which depend on the file size. You can bypass this\nusing `Membrane.File.Sink` to save the file or fixing the header afterwards with `Membrane.WAV.Postprocessing`\nif you need to use any other sink.","ref":"readme.html#warning"},{"type":"extras","title":"Sample usage - Membrane WAV Plugin","doc":"```elixir\ndefmodule Mixing.Pipeline do\n  use Membrane.Pipeline\n\n  @impl true\n  def handle_init(_ctx, _opts) do\n    spec = child(:file_src, %Membrane.File.Source{location: \"/tmp/input.wav\"})\n      |> child(:parser, Membrane.WAV.Parser)\n      |> child(:converter, %Membrane.FFmpeg.SWResample.Converter{\n        input_stream_format: %Membrane.RawAudio{channels: 1, sample_rate: 16_000, sample_format: :s16le},\n        output_stream_format: %Membrane.RawAudio{channels: 2, sample_rate: 48_000, sample_format: :s16le}\n      })\n      |> child(:serializer, Membrane.WAV.Serializer)\n      |> child(:file_sink, %Membrane.File.Sink{location: \"/tmp/output.wav\"})\n\n    {[spec: spec], %{}}\n  end\nend\n```","ref":"readme.html#sample-usage"},{"type":"extras","title":"Copyright and License - Membrane WAV Plugin","doc":"Copyright 2021, [Software Mansion](https://swmansion.com/?utm_source=git&utm_medium=readme&utm_campaign=membrane)\n\n[![Software Mansion](https://logo.swmansion.com/logo?color=white&variant=desktop&width=200&tag=membrane-github)](https://swmansion.com/?utm_source=git&utm_medium=readme&utm_campaign=membrane)\n\nLicensed under the [Apache License, Version 2.0](LICENSE)","ref":"readme.html#copyright-and-license"},{"type":"extras","title":"LICENSE","doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS\n\n   APPENDIX: How to apply the Apache License to your work.\n\n      To apply the Apache License to your work, attach the following\n      boilerplate notice, with the fields enclosed by brackets \"[]\"\n      replaced with your own identifying information. (Don't include\n      the brackets!)  The text should be enclosed in the appropriate\n      comment syntax for the file format. We also recommend that a\n      file or class name and description of purpose be included on the\n      same \"printed page\" as the copyright notice for easier\n      identification within third-party archives.\n\n   Copyright 2020 Software Mansion\n\n   Licensed under the Apache License, Version 2.0 (the \"License\");\n   you may not use this file except in compliance with the License.\n   You may obtain a copy of the License at\n\n       http://www.apache.org/licenses/LICENSE-2.0\n\n   Unless required by applicable law or agreed to in writing, software\n   distributed under the License is distributed on an \"AS IS\" BASIS,\n   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n   See the License for the specific language governing permissions and\n   limitations under the License.","ref":"license.html"}],"content_type":"text/markdown"}